PROGRAM_NAME='Lighting'
#include 'SNAPI';
#include 'G4API'

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

integer MAX_LIGHT_BUTTONS	= 3;
integer MAX_LIGHTS		= 4;

integer BTN_LIGHTS_ALL_ON	= 27;
integer BTN_LIGHTS_ALL_OFF	= 28;

integer BTN_LIGHTS_LEVEL	= 301;


(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

volatile integer nLvlLights[MAX_LIGHTS];
volatile sinteger nLightsScene;


DEFINE_FUNCTION updateLightsLevelFeedback()
{
    stack_var integer iLights;
    
    for(iLights = 1; iLights <= MAX_LIGHTS; iLights++) 
    {
	send_level dvTP_1_LIGHTS, iLights, nLvlLights[iLights];
    }
}

DEFINE_FUNCTION updateLightsPresetFeedback()
{
    stack_var integer iScene;
    
    for(iScene = 1; iScene <= MAX_LIGHT_BUTTONS; iScene++) 
    {
	[dvTP_1_LIGHTS, BTN_LIGHT_ELEMENT[iScene]] = (type_cast(nLightsScene) == iScene);
    }
    [dvTP_1_LIGHTS, BTN_LIGHTS_ALL_ON] = (nLightsScene == 255);
    [dvTP_1_LIGHTS, BTN_LIGHTS_ALL_OFF] = (nLightsScene == 0);

}

(***********************************************************)
(*                  THE EVENTS GO BELOW                    *)
(***********************************************************)
DEFINE_EVENT

button_event[dvTP_1_LIGHTS, BTN_LIGHTS_ALL_ON]
{
    push:
    {
	to[button.input];
	send_command vdvLIGHTS,"'SCENEPRESET-255'"
    }
    release:
    {
	updateLightsPresetFeedback();
    }
}

button_event[dvTP_1_LIGHTS, BTN_LIGHTS_ALL_OFF]
{
    push:
    {
	to[button.input];
	send_command vdvLIGHTS,"'SCENEPRESET-0'"
    }
    release:
    {
	updateLightsPresetFeedback();
    }
}


button_event[dvTP_1_LIGHTS, BTN_LIGHT_ELEMENT]
{
    push:
    {
	to[button.input];
    }
    hold[5]:
    {
	stack_var integer nBtnIdx;

	nBtnIdx = get_last(BTN_LIGHT_ELEMENT);
	send_command vdvLIGHTS,"'SCENESAVE-',itoa(nBtnIdx)"
	off[button.input];
    }
    release:
    {
	stack_var integer nBtnIdx;

	nBtnIdx = get_last(BTN_LIGHT_ELEMENT);
	send_command vdvLIGHTS,"'SCENEPRESET-',itoa(nBtnIdx)"
	updateLightsPresetFeedback();
    }
}

button_event[dvTP_1_LIGHTS, BTN_LIGHTS_LEVEL]
{
    push:
    {
	to[button.input];
    }
    release:
    {
	updateLightsLevelFeedback();
    }
}

level_event[dvTP_1_LIGHTS, 0]
{
    if([level.input.device, BTN_LIGHTS_LEVEL]) 	// Level is being adjusted
    {
	send_level vdvLIGHTS, level.input.level, level.value;
    }
}

data_event[dvTP_1_LIGHTS]
{
    online:
    {
	updateLightsLevelFeedback();
	updateLightsPresetFeedback();
    }
}

data_event[vdvLIGHTS]
{
    online:
    {
	send_command data.device,"'PROPERTY-IP_Address,',IP_ADDRESS_LIGHTS";
	send_command data.device,"'PROPERTY-Port,12345'";
	send_command data.device,'REINIT';
    }
    command:
    {
	stack_var char iCmd[DUET_MAX_CMD_LEN], iHdr[DUET_MAX_HDR_LEN], iParam[DUET_MAX_PARAM_LEN];
	
	iCmd = data.text;
	iHdr = DuetParseCmdHeader(iCmd);
	switch(iHdr)
	{
	    case 'SCENEPRESET':
	    {
		nLightsScene = atoi(DuetParseCmdParam(iCmd));
		updateLightsPresetFeedback();
	    }
	}
    }
}

level_event[vdvLIGHTS, 0]
{
    nLvlLights[level.input.level] = level.value;
    if(![dvTP_1_LIGHTS, BTN_LIGHTS_LEVEL]) 	// Level is not being adjusted
    {
	send_level dvTP_1_LIGHTS, level.input.level, level.value;
    }
}

