﻿//*********************************************************************
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 04/14/2020  AT: 09:39:27        *)
(***********************************************************)
//
//             AMX Resource Management Suite  (4.6.11)
//
//*********************************************************************
/*
 *  Legal Notice :
 *
 *     Copyright, AMX LLC, 2011
 *
 *     Private, proprietary information, the sole property of AMX LLC.  The
 *     contents, ideas, and concepts expressed herein are not to be disclosed
 *     except within the confines of a confidential relationship and only
 *     then on a need to know basis.
 *
 *     Any entity in possession of this AMX Software shall not, and shall not
 *     permit any other person to, disclose, display, loan, publish, transfer
 *     (whether by sale, assignment, exchange, gift, operation of law or
 *     otherwise), license, sublicense, copy, or otherwise disseminate this
 *     AMX Software.
 *
 *     This AMX Software is owned by AMX and is protected by United States
 *     copyright laws, patent laws, international treaty provisions, and/or
 *     state of Texas trade secret laws.
 *
 *     Portions of this AMX Software may, from time to time, include
 *     pre-release code and such code may not be at the level of performance,
 *     compatibility and functionality of the final code. The pre-release code
 *     may not operate correctly and may be substantially modified prior to
 *     final release or certain features may not be generally released. AMX is
 *     not obligated to make or support any pre-release code. All pre-release
 *     code is provided "as is" with no warranties.
 *
 *     This AMX Software is provided with restricted rights. Use, duplication,
 *     or disclosure by the Government is subject to restrictions as set forth
 *     in subparagraph (1)(ii) of The Rights in Technical Data and Computer
 *     Software clause at DFARS 252.227-7013 or subparagraphs (1) and (2) of
 *     the Commercial Computer Software Restricted Rights at 48 CFR 52.227-19,
 *     as applicable.
*/

MODULE_NAME= 'RmsDuetBssBlu50RPMMonitor'(DEV vdvRMS,
				      DEV vdvDeviceModule,
				      DEV dvMonitoredDevice)
(***********************************************************)
(* System Type : NetLinx                                   *)
(***********************************************************)
// This compiler directive is provided as a clue so that other include
// files can provide SNAPI specific behavior if needed.
#DEFINE SNAPI_MONITOR_MODULE;

#DEFINE HAS_FIXED_POWER
#DEFINE HAS_VOLUME
#DEFINE HAS_GAIN

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

CHAR MONITOR_NAME[]       = 'RMS Audio Mixer Monitor';
CHAR MONITOR_DEBUG_NAME[] = 'RmsDuetAudioMixMon';
CHAR MONITOR_VERSION[]    = '4.6.11';
CHAR MONITOR_ASSET_TYPE[] = '';
CHAR MONITOR_ASSET_NAME[] = '';

#IF_DEFINED HAS_VOLUME
    SLONG   METADATA_PROPERTY_VOL_LVL_MIN     = 0;
    SLONG   METADATA_PROPERTY_VOL_LVL_MAX     = 255;
    INTEGER METADATA_PROPERTY_VOL_LVL_STEP    = 1;
    SLONG   METADATA_PROPERTY_VOL_LVL_INIT    = 0;
    SLONG   METADATA_PROPERTY_VOL_LVL_RESET   = 0;
    CHAR    METADATA_PROPERTY_VOL_LVL_UNITS[] = '';
#END_IF

#IF_DEFINED HAS_GAIN
    SLONG   METADATA_PROPERTY_GAIN_LVL_MIN     = 0;
    SLONG   METADATA_PROPERTY_GAIN_LVL_MAX     = 255;
    INTEGER METADATA_PROPERTY_GAIN_LVL_STEP    = 1;
    SLONG   METADATA_PROPERTY_GAIN_LVL_INIT    = 0;
    SLONG   METADATA_PROPERTY_GAIN_LVL_RESET   = 0;
    CHAR    METADATA_PROPERTY_GAIN_LVL_UNITS[] = '';
#END_IF



// include RMS MONITOR COMMON AXI
#INCLUDE 'RmsMonitorCommon';

#INCLUDE 'RmsNlSnapiComponents';

(***********************************************************)
(*              DATA TYPE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_TYPE

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE
volatile dev vdvVolume;
volatile dev vdvGain;

(***********************************************************)
(*        SUBROUTINE/FUNCTION DEFINITIONS GO BELOW         *)
(***********************************************************)


(***********************************************************)
(* Name:  RegisterAsset                                    *)
(* Args:  RmsAsset asset data object to be registered .    *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        register this asset.                             *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION RegisterAsset(RmsAsset asset)
{
  // setup optional asset properties
  //asset.name        = 'My Custom Device';
  //asset.description = 'Asset Description Goes Here!';

  // perform registration of this asset
  RmsAssetRegisterDuetDevice(dvMonitoredDevice, vdvDeviceModule, asset);
}


(***********************************************************)
(* Name:  RegisterAssetParameters                          *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        register this asset's parameters to be monitored *)
(*        by RMS.                                          *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION RegisterAssetParameters()
{
  // This Duet-based asset monitoring module will
  // automatically register the default monitoring
  // parameters.  If you wish to extend the capabilities
  // and add additional monitored parameters, please
  // add them here.
  //Register all snapi HAS_xyz components
  RegisterAssetParametersSnapiComponents(assetClientKey);

  // submit all parameter registrations
  RmsAssetParameterSubmit(assetClientKey);
}


(***********************************************************)
(* Name:  SynchronizeAssetParameters                       *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        update/synchronize this asset parameter values   *)
(*        with RMS.                                        *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION SynchronizeAssetParameters()
{
  // This callback method is invoked when either the RMS server connection
  // has been offline or this monitored device has been offline from some
  // amount of time.   Since the monitored parameter state values could
  // be out of sync with the RMS server, we must perform asset parameter
  // value updates for all monitored parameters so they will be in sync.
  // Update only asset monitoring parameters that may have changed in value.

  // This Duet-based asset monitoring module will
  // automatically synchronize the default monitored
  // parameter values.  If you extended teh monitored
  // parameters with your own custom parameters, please
  // make sure to synchronize those parameter values here.
  //Synchronize all snapi HAS_xyz components
  IF(SynchronizeAssetParametersSnapiComponents(assetClientKey))
    RmsAssetParameterSubmit (assetClientKey)
}



(***********************************************************)
(* Name:  ResetAssetParameterValue                         *)
(* Args:  parameterKey   - unique parameter key identifier *)
(*        parameterValue - new parameter value after reset *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that an asset          *)
(*        parameter value has been reset by the RMS server *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION ResetAssetParameterValue(CHAR parameterKey[],CHAR parameterValue[])
{
  // if your monitoring module performs any parameter
  // value tracking, then you may want to update the
  // tracking value based on the new reset value
  // received from the RMS server.
}



(***********************************************************)
(* Name:  RegisterAssetMetadata                            *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        register this asset's metadata properties with   *)
(*        RMS.                                             *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION RegisterAssetMetadata()
{
  // This Duet-based asset monitoring module will
  // automatically register the default asset type metadata
  // properties.  If you wish to extend the capabilities
  // and add additional metadata properties, please
  // add them here.
  //Register all snapi HAS_xyz components
  RegisterAssetMetadataSnapiComponents(assetClientKey);

  RmsAssetMetadataSubmit(assetClientKey);
}


(***********************************************************)
(* Name:  SynchronizeAssetMetadata                         *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        update/synchronize this asset metadata properties *)
(*        with RMS if needed.                              *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION SynchronizeAssetMetadata()
{
  // This callback method is invoked when either the RMS server connection
  // has been offline or this monitored device has been offline from some
  // amount of time.   Traditionally, asset metadata is relatively static
  // information and thus does not require any synchronization of values.
  // However, this callback method does provide the opportunity to perform
  // any necessary metadata updates if your implementation does include
  // any dynamic metadata values.
  //Register all snapi HAS_xyz components
  IF(SynchronizeAssetMetadataSnapiComponents(assetClientKey))
    RmsAssetMetadataSubmit(assetClientKey);
}


(***********************************************************)
(* Name:  RegisterAssetControlMethods                      *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        register this asset's control methods with RMS.  *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION RegisterAssetControlMethods()
{
  // This Duet-based asset monitoring module will
  // automatically register the default asset type control
  // method.  If you wish to extend the capabilities
  // and add additional asset control methods, please
  // add them here.
  //Register all snapi HAS_xyz components
  RegisterAssetControlMethodsSnapiComponents(assetClientKey);

  // when done enqueuing all asset control methods and
  // arguments for this asset, we just need to submit
  // them to finalize and register them with the RMS server
  RmsAssetControlMethodsSubmit(assetClientKey);
}


(***********************************************************)
(* Name:  ExecuteAssetControlMethod                        *)
(* Args:  methodKey - unique method key that was executed  *)
(*        arguments - array of argument values invoked     *)
(*                    with the execution of this method.   *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it should         *)
(*        fullfill the execution of one of this asset's    *)
(*        control methods.                                 *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION ExecuteAssetControlMethod(CHAR methodKey[], CHAR arguments[])
STACK_VAR
  CHAR cValue1[RMS_MAX_PARAM_LEN]
  INTEGER nValue1
{
  DEBUG("'<<< EXECUTE CONTROL METHOD : [',methodKey,'] args=',arguments,' >>>'");

  cValue1 = RmsParseCmdParam(arguments)
  nValue1 = ATOI(cValue1)

  SWITCH(LOWER_STRING(methodKey))
  {
#IF_DEFINED HAS_VOLUME
    CASE 'volume.mute' :
    {
      SWITCH(nValue1)
      {
        CASE TRUE  :
        {
//          on[vdvDeviceModule,VOL_MUTE_ON]
          on[vdvVolume,VOL_MUTE_ON]
        }
        CASE FALSE :
        {
//          off[vdvDeviceModule,VOL_MUTE_ON]
          off[vdvVolume,VOL_MUTE_ON]
        }
      }
    }
    CASE 'volume.level' :
    {
//      SEND_LEVEL vdvDeviceModule, VOL_LVL, nValue1
      SEND_LEVEL vdvVolume, VOL_LVL, nValue1
    }
#END_IF

#IF_DEFINED HAS_GAIN
    CASE 'gain.mute' :
    {
      SWITCH(nValue1)
      {
        CASE TRUE  :
        {
//          on[vdvDeviceModule,GAIN_MUTE_ON]
          on[vdvGain,VOL_MUTE_ON]
        }
        CASE FALSE :
        {
//          off[vdvDeviceModule,GAIN_MUTE_ON]
          off[vdvGain,VOL_MUTE_ON]
        }
      }
    }
    CASE 'gain.level' :
    {
//      SEND_LEVEL vdvDeviceModule, GAIN_LVL, nValue1
      SEND_LEVEL vdvGain, VOL_LVL, nValue1
    }
#END_IF

    DEFAULT :
    {
    }
  }
}


(***********************************************************)
(* Name:  SystemPowerChanged                               *)
(* Args:  powerOn - boolean value representing ON/OFF      *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that the SYSTEM POWER  *)
(*        state has changed states.                        *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION SystemPowerChanged(CHAR powerOn)
{
  // optionally implement logic based on
  // system power state.
}


(***********************************************************)
(* Name:  SystemModeChanged                                *)
(* Args:  modeName - string value representing mode change *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that the SYSTEM MODE   *)
(*        state has changed states.                        *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION SystemModeChanged(CHAR modeName[])
{
  // optionally implement logic based on
  // newly selected system mode name.
}


(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START
{
    vdvVolume = vdvDeviceModule.NUMBER:2:vdvDeviceModule.SYSTEM;
    vdvGain   = vdvDeviceModule.NUMBER:3:vdvDeviceModule.SYSTEM;

    REBUILD_EVENT();
}

(***********************************************************)
(*                THE EVENTS GO BELOW                      *)
(***********************************************************)
DEFINE_EVENT


//
// (DUET DEVICE EVENT HANDLERS)
//
// handle events generated by the Duet device
// module.
//
DATA_EVENT[vdvDeviceModule]
{
  ONLINE:
  {
    // assign the 'RMS-Type' property with a value of
    // 'Asset' on the Duet device module.
    SEND_COMMAND vdvDeviceModule, "'PROPERTY-RMS-Type,Asset'"
  }
}

#IF_DEFINED HAS_VOLUME
//  CHANNEL_EVENT[vdvDeviceModule,VOL_MUTE_FB]
  CHANNEL_EVENT[vdvVolume,VOL_MUTE_FB]
  {
    ON :
    {
      IF(IsRmsReady())
      {
        SWITCH(CHANNEL.CHANNEL)
        {
          CASE VOL_MUTE_FB         :  RmsAssetParameterSetValue(assetClientKey, 'volume.mute'           , 'true' );
        }
      }
    }
    OFF :
    {
      IF(IsRmsReady())
      {
        SWITCH(CHANNEL.CHANNEL)
        {
          CASE VOL_MUTE_FB         :  RmsAssetParameterSetValue(assetClientKey, 'volume.mute'           , 'false' );
        }
      }
    }
  }

//  LEVEL_EVENT[vdvDeviceModule,VOL_LVL]
  LEVEL_EVENT[vdvVolume,VOL_LVL]
  {
    KeySetValue ('volume.level', ITOA(LEVEL.VALUE))

    IF(IsRmsReady())
      RmsAssetParameterSetValue(assetClientKey, 'volume.level', ITOA(LEVEL.VALUE) );
  }
#END_IF

#IF_DEFINED HAS_GAIN
//  CHANNEL_EVENT[vdvDeviceModule,GAIN_MUTE_FB]
  CHANNEL_EVENT[vdvGain,VOL_MUTE_FB]
  {
    ON :
    {
      IF(IsRmsReady())
      {
        SWITCH(CHANNEL.CHANNEL)
        {
//          CASE GAIN_MUTE_FB        :   RmsAssetParameterSetValue(assetClientKey, 'gain.mute'             , 'true' );
          CASE VOL_MUTE_FB        :   RmsAssetParameterSetValue(assetClientKey, 'gain.mute'             , 'true' );
        }
      }
    }
    OFF :
    {
      IF(IsRmsReady())
      {
        SWITCH(CHANNEL.CHANNEL)
        {
//          CASE GAIN_MUTE_FB        :  RmsAssetParameterSetValue(assetClientKey, 'gain.mute'             , 'false' );
          CASE VOL_MUTE_FB        :  RmsAssetParameterSetValue(assetClientKey, 'gain.mute'             , 'false' );
        }
      }
    }
  }

//  LEVEL_EVENT[vdvDeviceModule,GAIN_LVL]
  LEVEL_EVENT[vdvGain,VOL_LVL]
  {
    KeySetValue ('gain.level', ITOA(LEVEL.VALUE))

    IF(IsRmsReady())
      RmsAssetParameterSetValue(assetClientKey, 'gain.level'  , ITOA(LEVEL.VALUE) );
  }
#END_IF

(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM

(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
