﻿//*********************************************************************
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 12/11/2019  AT: 09:26:16        *)
(***********************************************************)
//
//             AMX Resource Management Suite  (4.6.7)
//
//*********************************************************************
/*
 *  Legal Notice :
 *
 *     Copyright, AMX LLC, 2011
 *
 *     Private, proprietary information, the sole property of AMX LLC.  The
 *     contents, ideas, and concepts expressed herein are not to be disclosed
 *     except within the confines of a confidential relationship and only
 *     then on a need to know basis.
 *
 *     Any entity in possession of this AMX Software shall not, and shall not
 *     permit any other person to, disclose, display, loan, publish, transfer
 *     (whether by sale, assignment, exchange, gift, operation of law or
 *     otherwise), license, sublicense, copy, or otherwise disseminate this
 *     AMX Software.
 *
 *     This AMX Software is owned by AMX and is protected by United States
 *     copyright laws, patent laws, international treaty provisions, and/or
 *     state of Texas trade secret laws.
 *
 *     Portions of this AMX Software may, from time to time, include
 *     pre-release code and such code may not be at the level of performance,
 *     compatibility and functionality of the final code. The pre-release code
 *     may not operate correctly and may be substantially modified prior to
 *     final release or certain features may not be generally released. AMX is
 *     not obligated to make or support any pre-release code. All pre-release
 *     code is provided "as is" with no warranties.
 *
 *     This AMX Software is provided with restricted rights. Use, duplication,
 *     or disclosure by the Government is subject to restrictions as set forth
 *     in subparagraph (1)(ii) of The Rights in Technical Data and Computer
 *     Software clause at DFARS 252.227-7013 or subparagraphs (1) and (2) of
 *     the Commercial Computer Software Restricted Rights at 48 CFR 52.227-19,
 *     as applicable.
*/
MODULE_NAME='RmsSVSiDeviceMonitor'(DEV vdvRMS,
				    DEV dvMonitoredDevice);
#include 'SNAPI';


(***********************************************************)
(* System Type : NetLinx                                   *)
(***********************************************************)
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

    vdvDeviceModule = DYNAMIC_VIRTUAL_DEVICE;//33001:1:0;

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

CHAR MONITOR_NAME[]       = 'RMS SVSi Device Monitor';
CHAR MONITOR_DEBUG_NAME[] = 'RmsSVSiDevMon';
CHAR MONITOR_VERSION[]    = '4.6.7';
CHAR MONITOR_ASSET_TYPE[] = 'Unknown';
CHAR MONITOR_ASSET_NAME[] = '';

// Custom Keys - referenced in uKeys key-value set and RMS asset component keys
CHAR KEY_ASSET_NAME[ ]		= 'RMS-Asset-Name';
CHAR KEY_ASSET_DESCRIPTION[ ]	= 'RMS-Asset-Description'
CHAR KEY_STREAM_NUMBER[ ] 	= 'stream.number';
CHAR KEY_AUDIO_STREAM[ ] 	= 'audio.stream.number';
CHAR KEY_DEVICE_TYPE[ ] 	= 'device.type';
CHAR KEY_IP_ADDRESS[ ] 		= 'ip.address';
CHAR KEY_MAX_POWER[ ] 		= 'power.draw';
CHAR KEY_POWER_CONSUME[ ] 	= 'asset.power.consumption';
CHAR KEY_LIVE_PLAY[ ] 		= 'live.play';
CHAR KEY_DPS[ ] 		= 'd.p.s';
CHAR KEY_REBOOT[ ] 		= 'device.reboot';
CHAR KEY_PASSTHRU[ ] 		= 'pass.through';



(***********************************************************)
(***********************************************************)
// Key Value stuff
DEFINE_CONSTANT
#warn 'Override default keyvalue settings in consideration of named key constants and stored value data requirements' 
    integer TOTAL_KEY_COUNT 	= 15;	
    integer KEY_LENGTH	 	= 25;
    integer VALUE_LENGTH	= 160;
    
#include 'KeyValue';

DEFINE_VARIABLE

    volatile _uKeys uKeys;
(***********************************************************)
(***********************************************************)

#DEFINE INCLUDE_RMS_EVENT_CUSTOM_COMMAND_CALLBACK // Enable RmsEventCustomCommand callback
#INCLUDE 'RmsMonitorCommon';

#include 'RmsCommon';


(***********************************************************)
(*              DATA TYPE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_TYPE

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE


(***********************************************************)
(*        SUBROUTINE/FUNCTION DEFINITIONS GO BELOW         *)
(***********************************************************)

(***********************************************************)
(* Name:  RegisterAsset                                    *)
(* Args:  RmsAsset asset data object to be registered .    *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        register this asset.                             *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION RegisterAsset(RmsAsset asset)
{
    stack_var dev_info_struct iDevInfo;

    DEVICE_INFO(dvMonitoredDevice, iDevInfo);
    
    // populate asset fields
    asset.clientKey = RmsDevToString(dvMonitoredDevice);
    asset.manufacturerName = 'AMX SVSi';
    asset.manufacturerUrl = 'http://www.amx.com';
    asset.modelName = iDevInfo.DEVICE_ID_STRING;
    asset.firmwareVersion = iDevInfo.VERSION;
    asset.serialNumber = stringTrim(iDevInfo.SERIAL_NUMBER);

#warn 'TODO: discriminate more device types - parameters, control methods, power consumption'	
    if(find_string(upper_string(asset.modelName),'ENC',1)) 
    {
	asset.assetType = RMS_ASSET_TYPE_DIGITAL_MEDIA_ENCODER;
    }
    else if(find_string(upper_string(asset.modelName),'DEC',1)) 
    {
	asset.assetType = RMS_ASSET_TYPE_DIGITAL_MEDIA_DECODER;
    }

#warn 'Overide default name and description with vdvRMS proxy command @<d:p:s>-RMS-Asset-xxxx,<data>'
    asset.name = keyGetOrDefault(uKeys, KEY_ASSET_NAME, asset.modelName);
    asset.description = keyGetOrDefault(uKeys, KEY_ASSET_DESCRIPTION, "'SVSI ',asset.assetType,' device addressed @ ',asset.clientKey");
    
    keyAdd(uKeys, KEY_DEVICE_TYPE, asset.assetType)

    // register the virtual device
    RmsAssetRegister(vdvDeviceModule, asset);
}


(***********************************************************)
(* Name:  RegisterAssetParameters                          *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        register this asset's parameters to be monitored *)
(*        by RMS.                                          *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION RegisterAssetParameters()
{

    RmsAssetOnlineParameterEnqueue (assetClientKey, GetOnlineSnapiValue(dvMonitoredDevice));
    
    RmsAssetParameterEnqueueDecimal(
					assetClientKey,
					KEY_POWER_CONSUME,
					'Power Consumption Rate',
					'Last reported power consumption rate for this asset.',
					RMS_ASSET_PARAM_TYPE_POWER_CONSUMPTION,
					atof(keyGetValue(uKeys, KEY_POWER_CONSUME)),
					0,
					30,
					'Watts',
					RMS_ALLOW_RESET_NO,
					0,
					RMS_TRACK_CHANGES_YES
				  );

    select
    {
	active(KeyGetValue(uKeys, KEY_DEVICE_TYPE) == RMS_ASSET_TYPE_DIGITAL_MEDIA_ENCODER):
	{
	    RmsAssetParameterEnqueueNumber  (
						assetClientKey,
						KEY_STREAM_NUMBER,
						'Stream Number',
						'Shows the current encoded audio & video stream number.',
						RMS_ASSET_PARAM_TYPE_NONE,
						atoi(KeyGetValue(uKeys, KEY_STREAM_NUMBER)),
						0,
						32765,
						'',
						RMS_ALLOW_RESET_NO,
						0,
						RMS_TRACK_CHANGES_NO
					    )
	      RmsAssetParameterEnqueueEnumeration (
							assetClientKey,
							KEY_LIVE_PLAY,
							'Host Play',
							'Shows the encoder host play status.',
							RMS_ASSET_PARAM_TYPE_NONE,
							atoi(KeyGetValue(uKeys, KEY_LIVE_PLAY)),
							'live|local',
							RMS_ALLOW_RESET_NO,
							'live',
							RMS_TRACK_CHANGES_YES
						    )
	}
	active(KeyGetValue(uKeys, KEY_DEVICE_TYPE) == RMS_ASSET_TYPE_DIGITAL_MEDIA_DECODER):
	{
	    RmsAssetParameterEnqueueNumber  (
						assetClientKey,
						KEY_STREAM_NUMBER,
						'Subscribed Video Stream Number',
						'Shows the current decoded video stream number.',
						RMS_ASSET_PARAM_TYPE_NONE,
						atoi(KeyGetValue(uKeys, KEY_STREAM_NUMBER)),
						0,
						32765,
						'',
						RMS_ALLOW_RESET_NO,
						0,
						RMS_TRACK_CHANGES_NO
					    );
	    RmsAssetParameterEnqueueNumber  (
						assetClientKey,
						KEY_AUDIO_STREAM,
						'Subscribed Audio Stream Number',
						'Shows the current decoded audio stream number. 0 = Audio Follow Video is enabled',
						RMS_ASSET_PARAM_TYPE_NONE,
						atoi(KeyGetValue(uKeys, KEY_AUDIO_STREAM)),
						0,
						32765,
						'',
						RMS_ALLOW_RESET_NO,
						0,
						RMS_TRACK_CHANGES_NO
					    );
	      RmsAssetParameterEnqueueEnumeration (
							assetClientKey,
							KEY_LIVE_PLAY,
							'Local Play',
							'Shows the decoder local play status.',
							RMS_ASSET_PARAM_TYPE_NONE,
							atoi(KeyGetValue(uKeys, KEY_LIVE_PLAY)),
							'live|local',
							RMS_ALLOW_RESET_NO,
							'live',
							RMS_TRACK_CHANGES_YES
						    )
	}
    }
    // submit all parameter registrations
    RmsAssetParameterSubmit(assetClientKey);
}


(***********************************************************)
(* Name:  SynchronizeAssetParameters                       *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        update/synchronize this asset parameter values   *)
(*        with RMS.                                        *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION SynchronizeAssetParameters()
{
    RmsAssetOnlineParameterUpdate(assetClientKey, GetOnlineSnapiValue(dvMonitoredDevice))
    RmsAssetParameterEnqueueSetValueDecimal(assetClientKey,KEY_POWER_CONSUME,atof(keyGetValue(uKeys, KEY_POWER_CONSUME)));
    
    select
    {
	active(KeyGetValue(uKeys, KEY_DEVICE_TYPE) == RMS_ASSET_TYPE_DIGITAL_MEDIA_ENCODER):
	{
	    RmsAssetParameterEnqueueSetValue(assetClientKey, KEY_STREAM_NUMBER, keyGetValue(uKeys, KEY_STREAM_NUMBER));
	}
	active(KeyGetValue(uKeys, KEY_DEVICE_TYPE) == RMS_ASSET_TYPE_DIGITAL_MEDIA_DECODER):
	{
	    RmsAssetParameterEnqueueSetValue(assetClientKey, KEY_STREAM_NUMBER, keyGetValue(uKeys, KEY_STREAM_NUMBER));
	    RmsAssetParameterEnqueueSetValue(assetClientKey, KEY_AUDIO_STREAM, keyGetValue(uKeys, KEY_AUDIO_STREAM));
	}
    }

    RmsAssetParameterSubmit (assetClientKey)
}



(***********************************************************)
(* Name:  ResetAssetParameterValue                         *)
(* Args:  parameterKey   - unique parameter key identifier *)
(*        parameterValue - new parameter value after reset *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that an asset          *)
(*        parameter value has been reset by the RMS server *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION ResetAssetParameterValue(CHAR parameterKey[], CHAR parameterValue[])
{
  // if your monitoring module performs any parameter
  // value tracking, then you may want to update the
  // tracking value based on the new reset value
  // received from the RMS server.
}



(***********************************************************)
(* Name:  RegisterAssetMetadata                            *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        register this asset's metadata properties with   *)
(*        RMS.                                             *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION RegisterAssetMetadata()
{
  // register all asset metadata properties now.
    RmsAssetMetadataEnqueueHyperlink(
					assetClientKey,
					KEY_IP_ADDRESS,
					'IP Address',
					keyGetValue(uKeys, KEY_IP_ADDRESS),
					keyGetValue(uKeys, KEY_IP_ADDRESS)
				    );
 
    RmsAssetMetadataEnqueueString(
					assetClientKey,
					KEY_DPS,
					'NetLinx Native D:P:S',
					"assetClientKey"
				    )
    
    RmsAssetMetadataSubmit(assetClientKey);
}


(***********************************************************)
(* Name:  SynchronizeAssetMetadata                         *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        update/synchronize this asset metadata properties *)
(*        with RMS if needed.                              *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION SynchronizeAssetMetadata()
{
    RmsAssetMetadataUpdateHyperlink(
				    assetClientKey,
				    KEY_IP_ADDRESS,
				    keyGetValue(uKeys, KEY_IP_ADDRESS),
				    keyGetValue(uKeys, KEY_IP_ADDRESS)
				);
    RmsAssetMetadataUpdateString (
				    assetClientKey,
				    KEY_DPS,
				    "assetClientKey"
				);
				
    RmsAssetMetadataSubmit(assetClientKey);
}


(***********************************************************)
(* Name:  RegisterAssetControlMethods                      *)
(* Args:  -none-                                           *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it is time to     *)
(*        register this asset's control methods with RMS.  *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION RegisterAssetControlMethods()
{
    RmsAssetControlMethodEnqueue(assetClientKey,KEY_REBOOT,'Reboot', 'Reboot the device');
    RmsAssetControlMethodArgumentBoolean(assetClientKey,KEY_REBOOT,0,'Reboot','Reboot the device',TRUE);
    
    RmsAssetControlMethodEnqueue(assetClientKey,KEY_PASSTHRU,'PassThru', 'Pass native API command to the device');
    RmsAssetControlMethodArgumentString(assetClientKey,KEY_PASSTHRU,0,'API Command','Enter command to send','');

    select
    {
	active(KeyGetValue(uKeys, KEY_DEVICE_TYPE) == RMS_ASSET_TYPE_DIGITAL_MEDIA_ENCODER):
	{
	    RmsAssetControlMethodEnqueue(assetClientKey,KEY_STREAM_NUMBER,'Stream Number', 'Set the encoding audio & video stream number');
	    RmsAssetControlMethodArgumentNumberEx(
							assetClientKey,
							KEY_STREAM_NUMBER,
							1,
							'Stream Number',
							'Set the stream number to encode audio & video.',
							atoi(keyGetValue(uKeys, KEY_STREAM_NUMBER)),
							1,
							32765,
							1
						    );
	}
	active(KeyGetValue(uKeys, KEY_DEVICE_TYPE) == RMS_ASSET_TYPE_DIGITAL_MEDIA_DECODER):
	{
	    RmsAssetControlMethodEnqueue(assetClientKey,KEY_STREAM_NUMBER,'Stream Subscribe', 'Set the audio & video stream subscription');
	    RmsAssetControlMethodArgumentNumberEx(
							assetClientKey,
							KEY_STREAM_NUMBER,
							1,
							'Video Stream',
							'Set the video stream number to subscribe.  Set to -1 to be uneffected, 0 for no stream',
							atoi(keyGetValue(uKeys, KEY_STREAM_NUMBER)),
							-1,
							32765,
							1
						    );
	    RmsAssetControlMethodArgumentBoolean(
							assetClientKey,
							KEY_STREAM_NUMBER,
							2,
							'Audio Follow Video',
							'Select to have the audio switch with the video',
							TRUE
						    );
	    RmsAssetControlMethodArgumentNumberEx(
							assetClientKey,
							KEY_STREAM_NUMBER,
							3,
							'Audio Stream',
							'Set the audio stream number to subscribe if different from the video.  Set to -1 to be uneffected',
							atoi(keyGetValue(uKeys, KEY_AUDIO_STREAM)),
							-1,
							32765,
							1
						    );
	}
    }
  
    RmsAssetControlMethodsSubmit(assetClientKey);
}


(***********************************************************)
(* Name:  ExecuteAssetControlMethod                        *)
(* Args:  methodKey - unique method key that was executed  *)
(*        arguments - array of argument values invoked     *)
(*                    with the execution of this method.   *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that it should         *)
(*        fullfill the execution of one of this asset's    *)
(*        control methods.                                 *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION ExecuteAssetControlMethod(CHAR methodKey[], CHAR arguments[])
{
    stack_var sinteger iParam;
    
    select
    {
	active(methodKey == KEY_REBOOT):
	{
	    send_command dvMonitoredDevice,'REBOOT';
	}
	active(methodKey == KEY_PASSTHRU):
	{
	    send_string dvMonitoredDevice,arguments;
	}
	active(methodKey == KEY_STREAM_NUMBER):
	{
	    select
	    {
		active(KeyGetValue(uKeys, KEY_DEVICE_TYPE) == RMS_ASSET_TYPE_DIGITAL_MEDIA_ENCODER):
		{
		    iParam = atoi(DuetParseCmdParam(arguments));
		    
		    if(ValidateStreamNumber(iParam))
		    {
			send_command dvMonitoredDevice,"'CO ',itoa(iParam)";
		    }
		}
		active(KeyGetValue(uKeys, KEY_DEVICE_TYPE) == RMS_ASSET_TYPE_DIGITAL_MEDIA_DECODER):
		{
		    // 1st parameter - Video Stream
		    iParam = atoi(DuetParseCmdParam(arguments));
		    
		    if(ValidateStreamNumber(iParam))
		    {
			send_command dvMonitoredDevice,"'CI ',itoa(iParam)";
		    }
		    
		    // 2nd Parameter - Audio Follow Video
		    if(atoi(DuetParseCmdParam(arguments)))
		    {
			if(ValidateStreamNumber(iParam))
			{
			    send_command dvMonitoredDevice,"'CA 0'";	 
			}
		    }
		    else
		    {
			// 3rd Parameter - Audio Stream
			iParam = atoi(DuetParseCmdParam(arguments));
			
			if(ValidateStreamNumber(iParam))
			{
			    send_command dvMonitoredDevice,"'CA ',itoa(iParam)";
			}
		    }
		}
	    }
	}
    }
}

(***********************************************************)
(* Name:  SystemPowerChanged                               *)
(* Args:  powerOn - boolean value representing ON/OFF      *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that the SYSTEM POWER  *)
(*        state has changed states.                        *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION SystemPowerChanged(CHAR powerOn)
{
  // optionally implement logic based on
  // system power state.
}


(***********************************************************)
(* Name:  SystemModeChanged                                *)
(* Args:  modeName - string value representing mode change *)
(*                                                         *)
(* Desc:  This is a callback method that is invoked by     *)
(*        RMS to notify this module that the SYSTEM MODE   *)
(*        state has changed states.                        *)
(*                                                         *)
(*        This method should not be invoked/called         *)
(*        by any user implementation code.                 *)
(***********************************************************)
DEFINE_FUNCTION SystemModeChanged(CHAR modeName[])
{
  // optionally implement logic based on
  // newly selected system mode name.
}

(***********************************************************)
(* Callback from RmsEventListener 			   *)
(* #DEFINE INCLUDE_RMS_EVENT_CUSTOM_COMMAND_CALLBACK       *)
(***********************************************************)
DEFINE_FUNCTION RmsEventCustomCommand(CHAR header[], CHAR data[])
{
    stack_var char iParam[DUET_MAX_PARAM_LEN];
    
    amx_log(AMX_INFO,"'RmsEventCustomCommand(',header,', ',data,')'");
    iParam = DuetParseCmdParam(data);
    if(header[1] == '@')
    {
	header = right_string(header, length_string(header) - 1); // Remove @ prefix
	if(header == RmsDevToString(dvMonitoredDevice) && find_string(iParam, 'RMS-Asset-', 1))
	{
	    keyAdd(uKeys, iParam, DuetParseCmdParam(data));
	    amx_log(AMX_DEBUG,"'keyGetValue(uKeys,',iParam,')=',keyGetValue(uKeys, iParam)");
	}
    }
}

define_function char ValidateStreamNumber(sinteger iData)
{
    stack_var char iReturn;
    
    iReturn = (iData > -1 && iData <= 32765  && !((iData % 256 == 0) || (iData % 256 == 255)));	// Ya, not sure how to fix the type cast warning here...
    amx_log(AMX_INFO,"__FILE__,'->ValidateStreamNumber(',itoa(iData),') = ',RmsBooleanString(iReturn)");
    
    return iReturn
}



(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

(***********************************************************)
(*                THE EVENTS GO BELOW                      *)
(***********************************************************)
DEFINE_EVENT

data_event[dvMonitoredDevice]
{
    online:
    {
	stack_var ip_address_struct ipAddress;
	stack_var dev_info_struct iDevInfo;

	get_ip_address(data.device, ipAddress)
	keyAdd(uKeys, KEY_IP_ADDRESS, ipAddress.IPADDRESS);
	
	DEVICE_INFO(dvMonitoredDevice, iDevInfo);
	
	select
	{
	    active(find_string(upper_string(iDevInfo.DEVICE_ID_STRING),'ENC',1)):
	    {
		send_command data.device,'?VIDOUT_OUTPUT';
		send_command data.device,'?LIVE_PLAY';
	    }
	    active(find_string(upper_string(iDevInfo.DEVICE_ID_STRING),'DEC',1)):
	    {
		send_command data.device,'?VIDIN_INPUT';
		send_command data.device,'?AUDIN_INPUT';
		send_command data.device,'?LIVE_PLAY';
	    }
	}
	
	select
	{
	    active(find_string(upper_string(iDevInfo.DEVICE_ID_STRING),'-N24',1)): // POE+ devices
	    {
		keyAdd(uKeys, KEY_MAX_POWER, '30'); 
	    }
	    active(TRUE):							 	// POE devices
	    {
		keyAdd(uKeys, KEY_MAX_POWER, '12.5'); 
	    }
	}
	
	on[vdvDeviceModule, DEVICE_COMMUNICATING];  // Force RMS update

	IF(IsRmsReady())
	{
	  RmsAssetOnlineParameterUpdate(assetClientKey, GetOnlineSnapiValue(dvMonitoredDevice));
  
	  RmsAssetMetadataUpdateHyperlink(
					    assetClientKey,
					    KEY_IP_ADDRESS,
					    keyGetValue(uKeys, KEY_IP_ADDRESS),
					    keyGetValue(uKeys, KEY_IP_ADDRESS)
					);
	}
    }
    offline:
    {
	off[vdvDeviceModule, POWER_ON]; 
	off[vdvDeviceModule, DEVICE_COMMUNICATING];  
	
	IF(IsRmsReady())
	{
	  RmsAssetOnlineParameterUpdate(assetClientKey, GetOnlineSnapiValue(dvMonitoredDevice))
	}
    }
    command:
    {
	stack_var char iCmd[DUET_MAX_CMD_LEN], iHeader[DUET_MAX_HDR_LEN], iParam[DUET_MAX_PARAM_LEN];
	
	on[vdvDeviceModule, POWER_ON]; 

	iCmd = data.text;
	iHeader = DuetParseCmdHeader(iCmd);
	
	switch(iHeader)
	{
	    case 'VIDIN_INPUT':
	    case 'VIDOUT_OUTPUT':
	    {
		keyAdd(uKeys, KEY_STREAM_NUMBER, DuetParseCmdParam(iCmd));
		RmsAssetParameterSetValueNumber( 	
						    assetClientKey,
						    KEY_STREAM_NUMBER,
						     atoi(KeyGetValue(uKeys, KEY_STREAM_NUMBER))
					       );
					       
		on[vdvDeviceModule, DATA_INITIALIZED];  // Force RMS registration
	    }
	    case 'AUDIN_INPUT':
	    {
		keyAdd(uKeys, KEY_AUDIO_STREAM, DuetParseCmdParam(iCmd));
		RmsAssetParameterSetValueNumber( 	
						    assetClientKey,
						    KEY_AUDIO_STREAM,
						     atoi(KeyGetValue(uKeys, KEY_AUDIO_STREAM))
					       );
	    }
	    case 'LIVE_PLAY':
	    {
		keyAdd(uKeys, KEY_LIVE_PLAY, DuetParseCmdParam(iCmd));
		RmsAssetParameterSetValue(	
						assetClientKey,
						KEY_LIVE_PLAY,
						keyGetValue(uKeys, KEY_LIVE_PLAY)
					    );
	    }
	}
    }
}

channel_event[vdvDeviceModule, POWER_ON]
{
    on:
    {
 	keyAdd(uKeys, KEY_POWER_CONSUME, keyGetValue(uKeys, KEY_MAX_POWER)); 
	
   	IF(IsRmsReady())
	{
	  RmsAssetParameterSetValueDecimal(assetClientKey,KEY_POWER_CONSUME,atof(keyGetValue(uKeys, KEY_POWER_CONSUME)));
	}
}
    off:
    {
	keyAdd(uKeys, KEY_POWER_CONSUME, '0'); 

   	IF(IsRmsReady())
	{
	  RmsAssetParameterSetValueDecimal(assetClientKey,KEY_POWER_CONSUME,atof(keyGetValue(uKeys, KEY_POWER_CONSUME)));
	}
    }
}
(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM

(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
