PROGRAM_NAME='System'
#include 'RmsApi';

(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 04/13/2020  AT: 10:24:32        *)
(***********************************************************)

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

#WARN 'Set source constants as per connections at DVX'
integer INPUT_ROKU 	= 3;
integer INPUT_APPLE 	= 4;
integer INPUT_ENZO	= 5;
integer INPUT_LAPTOP	= 6;


(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

    volatile integer nInput;
    volatile integer nVolLvl;
    
    volatile long lTlSystemFb[ ] = {500};


DEFINE_FUNCTION SetSystemPower(char iState) 
{
    send_string 0,"'SetSystemPower(',itoa(iState),')'";
    set_pulse_time(5);
    if(iState == true) 
    {
	cancel_wait 'Lag Off';
	off[dvDVX_SWITCHER_IO_1, DVX_STANDBY];
	pulse[vdvPROJECTOR, PWR_ON];
	
	send_command vdvLIGHTS, 'SCENEPRESET-255';
	
	pulse[dvDVX_REL, SCREEN_DOWN];
	
	on[dvDVX_REL, LIFT_DOWN];
	wait 70
	{
	    off[dvDVX_REL, LIFT_DOWN];
	}
	
	on[dvDVX_IO, SHADES_CLOSE]
	wait 200
	{
	    off[dvDVX_IO, SHADES_CLOSE];
	}
	
	RmsSystemPowerOn();
    }
    else 
    {
	pulse[vdvPROJECTOR, PWR_OFF];
	
	send_command dvENZO,"'EXIT'";
	
	send_command dvDVX_SWITCHER_IO_1,"'VI0O1'";
	send_command dvDVX_SWITCHER_IO_1,"'AI0O1'";

	pulse[dvDVX_REL, SCREEN_UP];
	
	on[dvDVX_REL, LIFT_UP];
	wait 70
	{
	    off[dvDVX_REL, LIFT_UP];
	}
	
	on[dvDVX_IO, SHADES_OPEN]
	wait 200
	{
	    off[dvDVX_IO, SHADES_OPEN];
	}
	
	wait 600 'Lag Off'
	{
	    send_command vdvLIGHTS, 'SCENEPRESET-0';
	    on[dvDVX_SWITCHER_IO_1, DVX_STANDBY];
	}
	
	RmsSourceUsageDeactivateAllSources();
	RmsSystemPowerOff();
    }
}

DEFINE_FUNCTION setSystemInput(integer iInput)
{
    send_string 0,"'setSystemInput(',itoa(iInput),')'";
    if(!nInput)
    {
	setSystemPower(true);
    }
    wait_until (![dvDVX_SWITCHER_IO_1, DVX_STANDBY])
    {
	send_command dvDVX_SWITCHER_IO_1,format('VI%dO1', iInput);
	send_command dvDVX_SWITCHER_IO_1,format('AI%dO1', iInput);
	off[dvDVX_SWITCHER_IO_1, VOL_MUTE_ON];
	if([vdvPROJECTOR, POWER_FB]) 
	{
	    send_command vdvPROJECTOR,'INPUTSELECT-1';
	}
    }

    switch(iInput)
    {
	case INPUT_ROKU:
	case INPUT_APPLE:
	{
	    RmsSystemSetMode('Entertainment');
	}
	case INPUT_ENZO:
	{
	    RmsSystemSetMode('Conference');
	}
	case INPUT_LAPTOP:
	{
	    RmsSystemSetMode('Presentation');
	}
    }

}

DEFINE_FUNCTION updateSystemFeedback()
{
    [dvTP_1, POWER_FB] 	= ([vdvPROJECTOR, POWER_ON] && ![dvDVX_SWITCHER_IO_1, DVX_STANDBY]);

    [dvTP_1,BTN_INPUT_SOURCE[1]] = ([dvTP_1, POWER_FB] && nInput == INPUT_ROKU);	
    [dvTP_1,BTN_INPUT_SOURCE[2]] = ([dvTP_1, POWER_FB] && nInput == INPUT_APPLE);	
    [dvTP_1,BTN_INPUT_SOURCE[3]] = ([dvTP_1, POWER_FB] && nInput == INPUT_ENZO);	
    [dvTP_1,BTN_INPUT_SOURCE[4]] = ([dvTP_1, POWER_FB] && nInput == INPUT_LAPTOP);	

    [dvTP_1, BTN_VOL_MUTE] = (![dvDVX_SWITCHER_IO_1, DVX_STANDBY] && [dvDVX_SWITCHER_IO_1, VOL_MUTE_FB]);
    if(![dvDVX_SWITCHER_IO_1, DVX_STANDBY])
    {
	send_level dvTP_1, VOL_LVL, nVolLvl;
    }
    else 
    {
	send_level dvTP_1, VOL_LVL, 0;
    }
    
    // Rms Source Tracking
    [vdvRMSSourceUsage, 1] = (nInput == INPUT_ROKU);	
    [vdvRMSSourceUsage, 2] = (nInput == INPUT_APPLE);	
    [vdvRMSSourceUsage, 3] = (nInput == INPUT_ENZO);	
    [vdvRMSSourceUsage, 4] = (nInput == INPUT_LAPTOP);	
}


(***********************************************************)
(*                  THE EVENTS GO BELOW                    *)
(***********************************************************)
DEFINE_EVENT

button_event[dvTP_1, 0]
{
    push:
    {
	to[button.input];
	switch(button.input.channel)
	{
	    case PWR_ON:
	    {
		setSystemPower(true);
		send_string dvRS232_CONFIDENCE,"'PWR 1',$0D"; // Power On
	    }
	    case PWR_OFF:
	    {
		setSystemPower(false);
		send_string dvRS232_CONFIDENCE,"'PWR 0',$0D"; // Power Off
	    }
	    case POWER_ON:
	    {
		if(nInput)
		{
		    setSystemPower(false);
		}
		else
		{
		    setSystemPower(true);
		}
	    }
	    case BTN_VOL_UP:
	    case BTN_VOL_DN:
	    case BTN_VOL_MUTE:
	    {
		to[dvDVX_SWITCHER_IO_1, button.input.channel];
	    }
	}
    }
}

button_event[dvTP_1, BTN_INPUT_SOURCE]
{
    push:
    {
	stack_var integer iSource;
	
	to[button.input];
	iSource =get_last(BTN_INPUT_SOURCE);
	switch(iSource)
	{
	    case 1:	
	    {
		setSystemInput(INPUT_ROKU);
	    }
	    case 2: 	
	    {
		setSystemInput(INPUT_APPLE);
	    }
	    case 3:	
	    {
		setSystemInput(INPUT_ENZO);
	    }
	    case 4:	
	    {
		setSystemInput(INPUT_LAPTOP);
	    }
	}
    }
}

channel_event[dvDVX_SWITCHER_IO_1, DVX_STANDBY]
{
    off:
    {
	send_command channel.device, '?INPUT-VIDEO,1';	 // Query current video source to set button feedback
    }
}

data_event[dvDVX_SWITCHER_IO_1]
{
    online:
    {
	send_command data.device, '?INPUT-VIDEO,1';	// Query current video source to set button feedback
    }
    command:
    {
	stack_var char iCmd[DUET_MAX_CMD_LEN], iHeader[DUET_MAX_HDR_LEN], iParam[DUET_MAX_PARAM_LEN];
	
	iCmd = data.text;
	iHeader = DuetParseCmdHeader(iCmd);
	iParam = DuetParseCmdParam(iCmd);
	switch(iHeader)
	{
	    case 'SWITCH':
	    {
		if(compare_string('LVIDEOI?O1', iParam))	// Only care about video input for system source tracking
		{
		    nInput = atoi(iParam);
		}
	    }
	}
    }
}

level_event[dvDVX_SWITCHER_IO_1, VOL_LVL]
{
    nVolLvl = level.value;
}

channel_event[vdvPROJECTOR, LAMP_WARMING_FB]
{
    off:
    {
	if([vdvPROJECTOR, LAMP_POWER_FB])
	{
	    send_command vdvPROJECTOR,'INPUTSELECT-1';	// Force HDMI input
	}
    }
}

data_event[dvTP_1]
{
    online:
    {
	if(!timeline_active(TL_SYSTEM_FB)) 
	{
	    timeline_create(TL_SYSTEM_FB, lTlSystemFb, length_array(lTlSystemFb), TIMELINE_ABSOLUTE, TIMELINE_REPEAT);
	}
    }
    offline:
    {
 	if(timeline_active(TL_SYSTEM_FB)) 
	{
	    timeline_kill(TL_SYSTEM_FB);
	}
   }
}

timeline_event[TL_SYSTEM_FB]
{
    updateSystemFeedback();
}

